from django.conf.urls import include, url
from django.contrib.auth.decorators import login_required
from . import views

urlpatterns = [

    url(r'^$',
        login_required(views.UserPreferenceFormView.as_view()),
        name="user"),
    url(r'^(?P<section>[\w\ ]+)$',
        login_required(views.UserPreferenceFormView.as_view()),
        name="user.section"),
]



from django.contrib import admin
from django import forms

from .settings import preferences_settings
from .registries import global_preferences_registry
from .models import GlobalPreferenceModel
from .forms import GlobalSinglePreferenceForm, SinglePerInstancePreferenceForm
from django.utils.translation import gettext_lazy as _

class SectionFilter(admin.AllValuesFieldListFilter):

    def __init__(self, field, request, params, model, model_admin, field_path):
        super(SectionFilter, self).__init__(field, request, params, model, model_admin, field_path)
        parent_model, reverse_path = admin.utils.reverse_field_path(model, field_path)
        if model == parent_model:
            queryset = model_admin.get_queryset(request)
        else:
            queryset = parent_model._default_manager.all()
        self.registries = []
        registry_name_set = set()
        for preferenceModel in queryset.distinct():
            l = len(registry_name_set)
            registry_name_set.add(preferenceModel.registry.__class__.__name__)
            if len(registry_name_set) != l:
                self.registries.append(preferenceModel.registry)


    def choices(self, changelist):
        choices = super(SectionFilter, self).choices(changelist)
        for choice in choices:
            display = choice['display']
            try:
                for registry in self.registries:
                    display = registry.section_objects[display].verbose_name
                choice["display"] = display
            except (KeyError):
                pass
            yield choice


class DynamicPreferenceAdmin(admin.ModelAdmin):
    list_display = ('verbose_name', 'name', 'section_name', 'help_text', 'raw_value', 'default_value')
    fields = ('raw_value', 'default_value', 'name', 'section_name')
    readonly_fields = ('name', 'section_name', 'default_value')
    if preferences_settings.ADMIN_ENABLE_CHANGELIST_FORM:
        list_editable = ('raw_value',)
    search_fields = ['name', 'section', 'raw_value']
    list_filter = (('section',  SectionFilter),)

    if preferences_settings.ADMIN_ENABLE_CHANGELIST_FORM:
        def get_changelist_form(self, request, **kwargs):
            return self.changelist_form

    def default_value(self, obj):
        return obj.preference.default
    default_value.short_description = _("Default Value")

    def section_name(self, obj):
        try:
            return obj.registry.section_objects[obj.section].verbose_name
        except KeyError:
            pass
        return obj.section

    section_name.short_description = _("Section Name")

class GlobalPreferenceAdmin(DynamicPreferenceAdmin):
    form = GlobalSinglePreferenceForm
    changelist_form = GlobalSinglePreferenceForm

    def get_queryset(self, *args, **kwargs):
        # Instanciate default prefs
        manager = global_preferences_registry.manager()
        manager.all()
        return super(GlobalPreferenceAdmin, self).get_queryset(*args, **kwargs)

admin.site.register(GlobalPreferenceModel, GlobalPreferenceAdmin)


class PerInstancePreferenceAdmin(DynamicPreferenceAdmin):
    list_display = ('instance',) + DynamicPreferenceAdmin.list_display
    fields = ('instance',) + DynamicPreferenceAdmin.fields
    raw_id_fields = ('instance',)
    form = SinglePerInstancePreferenceForm
    changelist_form = SinglePerInstancePreferenceForm
    list_select_related = True



from ..registries import PerInstancePreferenceRegistry


class UserPreferenceRegistry(PerInstancePreferenceRegistry):
    section_url_namespace = 'dynamic_preferences:user.section'


user_preferences_registry = UserPreferenceRegistry()



from django.apps import AppConfig, apps
from django.conf import settings
from django.utils.translation import gettext_lazy as _
from .registries import preference_models, global_preferences_registry


class DynamicPreferencesConfig(AppConfig):
    name = 'dynamic_preferences'
    verbose_name = _("Dynamic Preferences")

    def ready(self):
        GlobalPreferenceModel = self.get_model('GlobalPreferenceModel')

        preference_models.register(
            GlobalPreferenceModel, global_preferences_registry)

        # This will load all dynamic_preferences_registry.py files under
        # installed apps
        app_names = [app.name for app in apps.app_configs.values()]
        global_preferences_registry.autodiscover(app_names)



from django.apps import AppConfig, apps
from django.conf import settings
from django.utils.translation import gettext_lazy as _

from ..registries import preference_models
from .registries import user_preferences_registry


class UserPreferencesConfig(AppConfig):
    name = 'dynamic_preferences.users'
    verbose_name = _("Preferences - Users")
    label = 'dynamic_preferences_users'

    def ready(self):
        UserPreferenceModel = self.get_model('UserPreferenceModel')

        preference_models.register(
            UserPreferenceModel, user_preferences_registry)



# Generated by Django 2.0.6 on 2018-06-15 16:20

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='UserPreferenceModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('section', models.CharField(blank=True, db_index=True, default=None, max_length=150, null=True)),
                ('name', models.CharField(db_index=True, max_length=150)),
                ('raw_value', models.TextField(blank=True, null=True)),
                ('instance', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'user preference',
                'verbose_name_plural': 'user preferences',
                'abstract': False,
            },
        ),
        migrations.AlterUniqueTogether(
            name='userpreferencemodel',
            unique_together={('instance', 'section', 'name')},
        ),
    ]



# Generated by Django 2.1.7 on 2019-10-01 14:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dynamic_preferences', '0005_auto_20181120_0848'),
    ]

    operations = [
        migrations.AlterField(
            model_name='globalpreferencemodel',
            name='section',
            field=models.CharField(blank=True, db_index=True, default=None, max_length=150, null=True, verbose_name='Section Name'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dynamic_preferences', '0004_move_user_model'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='globalpreferencemodel',
            options={'verbose_name': 'Global preference', 'verbose_name_plural': 'Global preferences'},
        ),
        migrations.AlterField(
            model_name='globalpreferencemodel',
            name='name',
            field=models.CharField(db_index=True, max_length=150, verbose_name='Name'),
        ),
        migrations.AlterField(
            model_name='globalpreferencemodel',
            name='raw_value',
            field=models.TextField(blank=True, null=True, verbose_name='Raw Value'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('dynamic_preferences', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='globalpreferencemodel',
            name='name',
            field=models.CharField(max_length=150, db_index=True),
        ),
        migrations.AlterField(
            model_name='globalpreferencemodel',
            name='section',
            field=models.CharField(max_length=150, blank=True, db_index=True, default=None, null=True),
        ),
    ]



from rest_framework import serializers
from dynamic_preferences.models import GlobalPreferenceModel


class PreferenceValueField(serializers.Field):
    def get_attribute(self, o):
        return o

    def to_representation(self, o):
        return o.preference.api_repr(
            o.value
        )

    def to_internal_value(self, data):
        return data


class PreferenceSerializer(serializers.Serializer):

    section = serializers.CharField(read_only=True)
    name = serializers.CharField(read_only=True)
    identifier = serializers.SerializerMethodField()
    default = serializers.SerializerMethodField()
    value = PreferenceValueField()
    verbose_name = serializers.SerializerMethodField()
    help_text = serializers.SerializerMethodField()
    additional_data = serializers.SerializerMethodField()
    field = serializers.SerializerMethodField()

    class Meta:
        fields = [
            'default',
            'value',
            'verbose_name',
            'help_text',
        ]

    def get_default(self, o):
        return o.preference.api_repr(
            o.preference.get('default')
        )

    def get_verbose_name(self, o):
        return o.preference.get('verbose_name')

    def get_identifier(self, o):
        return o.preference.identifier()

    def get_help_text(self, o):
        return o.preference.get('help_text')

    def get_additional_data(self, o):
        return o.preference.get_api_additional_data()

    def get_field(self, o):
        return o.preference.get_api_field_data()

    def validate_value(self, value):
        """
        We call validation from the underlying form field
        """
        field = self.instance.preference.setup_field()
        value = field.to_python(value)
        field.validate(value)
        field.run_validators(value)
        return value

    def update(self, instance, validated_data):
        instance.value = validated_data['value']
        instance.save()
        return instance


class GlobalPreferenceSerializer(PreferenceSerializer):
    pass



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('dynamic_preferences', '0002_auto_20150712_0332'),
    ]

    operations = [
        migrations.AlterField(
            model_name='globalpreferencemodel',
            name='name',
            field=models.CharField(max_length=150, db_index=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='globalpreferencemodel',
            name='section',
            field=models.CharField(max_length=150, null=True, default=None, db_index=True, blank=True, verbose_name='Section Name'),
            preserve_default=True,
        ),
    ]
